C
C                     densities-micro.for
C***************************************************************************
C
C   MICROSCOPICAL STATISTICAL CALCULATION OF THE TOTAL STATE AND LEVEL
C   DENSITIES. COLLECTIVE ENHANCEMENT is CONSIDERED PHENOMENOLOGICALLY
C
C     AUTHOR: Dr.ANATOLY V.IGNATYUK
C             Institute of Physics and Power Engineering,
C             Bondarenko Sq.1, 249020 Obninsk, Kaluga region, RUSSIA
C             INTERNET:ignatyuk@cjd.obninsk.su
C             FAX:     +7-095-2302326
C
C  NOTE: MODIFIED AND TESTED BY R.CAPOTE 01/2002 in order to use SPL file
C  readed from MOELLER-NIX7HFBCS database (single-particle-levels directory)
C  by retrieve.for code.
C  For any question or comment related to the calculations, please
C  contact Prof.Ignatyuk directly.
C
C  The code was developed and tested on PC computer running WINDOWS
C              with Microsoft FORTRAN Power Station.
C  It should run without problems under UNIX, but it was not tested.
C
C--------------------------------------------------------------------------
C
C  You should prepare an INPUT FILE as well as file, containing
C  shell model SPL. The SPL files are provided by the code moller_retr.for
C  which is included in the Supplement OF THE RIPL.
C
C-----------------------------------------------------------------------------
C  SAMPLE INPUT FILE: SN120M.inp
Cline#1:  Z050A120.SPE             Shell Model input spectrum filename
C                                  This is an output file of retrieval-SPL.for
Cline#2:   GZ   GN  TMAX TMIN TSTEP         COMMENT line
Cline#3:  18.7  12. 4.2   .4  .2
C-----------------------------------------------------------------------------
C  COMMENTS: 1) The shell model input spectra file is provided by
C      moller_retr.for code, which extract single-particle levels from the
C      Moeller-Nix SPL database(file moller_levels.dat included in the
C      SUPPLEMENT section of the RIPL)
C            2) GZ and GN are the pairing constants for neutron and proton
C      system respectively
C            3) TMAX, TMIN and TSTEP are maximum, minimum and temperature
C      step for the density calculations
C ----------------------------------------------------------------------------
C   Using this input file you will get the following output file: BCSCOL.OUT
C       The level and state density for each energy as well as collective
C       enhancement factors for a given temperature and excitation energy
C       are tabulated.
C

      PROGRAM BCSR
      PARAMETER(IDIM=300)
      DIMENSION EPSZ(IDIM),IPSZ(IDIM),EPSN(IDIM),IPSN(IDIM),
     > RESZ(25),RESN(25),RES(25)
      COMMON/TEMP/TMIN,TMAX,TSTEP,Z,A,GZ,GN
      COMMON BET
      PI=3.1415926

      CALL SPL(EPSZ,IPSZ,NLEVZ,EPSN,IPSN,NLEVN)
      IF(TSTEP.EQ.0.) stop 'Temperature step must be greater than 0 !!'
      IF((TMAX-TMIN)/TSTEP.gt.25) stop 'MORE THAN 25 TEMPERATURE STEPS'

      OPEN(20,FILE='bcscol.out')
      write(20,1001) Z,A,BET
      write(20,5001) GZ,GN
      write(*,1001) Z,A,BET
      write(*,5001) GZ,GN
 5001 FORMAT(/,20X,'FORCES  GZ =' ,F10.5,3X,'GN =',F10.5)
 1001 FORMAT(/,10X,'ELEMENT Z=',F6.3,3X,'MASS NUMBER A=',F9.3,3X,'DEFORM
     >ATION BETA=',F6.4/)
      DO 1 K=1,25
      RESZ(K)=0.
      RESN(K)=0.
    1 CONTINUE
      RESZ(1)=Z
      RESN(1)=A-Z
      RESZ(2)=GZ/A
      RESN(2)=GN/A
      write(20,1112)
      write(*,1112)
 1112 FORMAT(//,4X,'Z/N',10X,
     C'G',8X,'LAM0',6X,'DEL0',6X,'TCR',6X,'ECOND',7X,
     C'E',8X,'D2N')
      CALL BCS(EPSZ,IPSZ,RESZ,NLEVZ)
      write(20,1010) (RESZ(L),L=1,8)
      write(*,1010) (RESZ(L),L=1,8)
      CALL BCS(EPSN,IPSN,RESN,NLEVN)
      write(20,1010) (RESN(L),L=1,8)
      write(*,1010) (RESN(L),L=1,8)
 1010 FORMAT(6F10.4,2X,2F10.4)

      TEMP=TMIN
      PRINT 2112
      write(20,1111)
    2 RESZ(9)=TEMP
      RESN(9)=TEMP
      CALL BCS(EPSZ,IPSZ,RESZ,NLEVZ)
      CALL BCS(EPSN,IPSN,RESN,NLEVN)
c 1101 FORMAT(2X,2F5.2,2X,5(E12.5,3X),/,8(E9.3,1X))
c      PRINT 1101,(RESZ(L),L=9,22)
c      write(20,1101)(RESZ(L),L=9,22)
c      PRINT 1101,(RESN(L),L=9,22)
c      write(20,1101)(RESN(L),L=9,22)
      RES(1)=Z
      RES(2)=A
      RES(3)=TEMP
      RES(4)=RESZ(12)+RESN(12)
      RES(5)=RESZ(13)+RESN(13)
      RES(6)=RES(5)-ALOG((2.*PI)**1.5*SQRT(RESZ(14)*
     >RESN(18)+RESZ(18)*RESN(14)))
      RES(7)=RESZ(15)+RESN(15)
      RES(8)=RESZ(19)+RESN(19)
      RES(9)=RES(8)/TEMP
      RES(10)=PI**2*RES(7)/6./TEMP
      RES(11)=RES(5)**2/4./(RES(4)-RESZ(6)-RESN(6))
      RES(12)=(RES(4)-RESZ(6)-RESN(6))/TEMP**2
      RES(13)=RES(5)/2./TEMP
      RES(14)=RES(8)/RES(7)
C***  VIBRATIONAL ENHANCEMENT IS ADDED FOR BETA&GAMMA MODES
C***  WITH OM-BETA=OM-GAMMA=OM2-SYSTEMATICS
      OM2=30./A**.66667
      CALL QVIBR(TEMP,OM2,A,3,Q2)
C***  ROTATIONAL ENHANCEMENT IS ADDED IN ADIABATIC APPROXIMATION
      U=RES(4)
      SIG=RESZ(22)+RESN(22)
      CALL QROT(A,BET,SIG,U,QR)
      COL=ALOG(Q2*QR)
      RES(6)=RES(6)+COL
      RES(5)=RES(6)-.5*ALOG(2.*PI*RES(8))
C***  RES(6) IS THE STATE DENSITY AND RES(5) IS THE LEVEL DENSITY
      TEMP=TEMP+TSTEP
      write(20,1104) RES(3),Q2,QR,(RES(K),K=4,14)
      write(*,1104) RES(3),Q2,QR,(RES(K),K=4,9)
 1104 FORMAT(2X,F4.2,2(F6.2),1X,4(F6.2),1X,F6.2,1X,F5.1,5(F7.2))
 1111 FORMAT(/,4X,'T',3X,
     >'QVIB',2X,'QROT',5X,'U',3X,'LnRO',2X,'LnST',2X,
     >'D2A',4X,
     >'SI2',4X,'F||',4X,'A',5X,'A1',5X,'A2',5X,'A3',
     >5X,'M2'/)
 2112 FORMAT(/,4X,'T',3X,
     >'QVIB',2X,'QROT',5X,'U',3X,'LnRO',2X,'LnST',2X,
     >'D2A',4X,
     >'SI2',4X,'F||')

      IF(TEMP-TMAX) 2,2,3
    3 CLOSE(20)
      STOP
      END
C
      SUBROUTINE BCS(EPS,IPS,RES,NLEV)
      PARAMETER(IDIM=300)
      DIMENSION EPS(IDIM),IPS(IDIM),RES(25),DF(3),DDF(5)
      COMMON BET
      PI=3.1415926
      FF1=RES(1)
      FF2=2./RES(2)
      EBL=0.
      IF(RES(7)) 25,10,25
   10 E0=0.
      EN=0.
      I=0
   11 I=I+1
      FJ=2.
      E0=E0+FJ*EPS(I)
      EN=EN+FJ
      IF(EN-FF1) 11,12,12
   12 E0=E0-(EN-FF1)*EPS(I)
      DLA=EPS(I)+0.01
      DEL=1.
      TEMP=0.
      NN=FF1
      NNN=2*(NN/2)
      IF(NN.EQ.NNN) GOTO 13
      EBL=EPS(I)
      IBL=I
      FF1=FF1-1.
   13 MET1=0
      MET2=1
      GOTO 30
   15 RES(3)=DLA
      RES(4)=DEL
      MET=5
      ASSIGN 211 TO METKA
      GOTO 100
  211 RES(7)=DF(1)-DEL**2/RES(2)+EBL
      RES(8)=DF(3)
      RES(6)=E0-RES(7)
      IF(RES(6)) 1000,1001,1001
 1000 EBL=EPS(IBL+1)
      GOTO 13
 1001 IF(DEL-1.E-2) 16,17,17
   16 RES(5)=0.
      RETURN
   17 TEMP=.567*RES(4)
      DLA=RES(3)
      DEL=0.
      MET1=2
      MET2=2
      GOTO 30
   20 RES(5)=TEMP
      RETURN
   25 TEMP=RES(9)
      DLA=RES(3)
      TFR=RES(5)
      IF(TEMP-TFR) 26,40,40
   26 DEL=2.*TFR*SQRT(1.-TEMP/TFR)
      MET1=0
      MET2=3
      GOTO 30
   30 MET=MET1+1
      ASSIGN 201 TO METKA
      GOTO 100
  201 DO 31 I=1,3
   31 DDF(I)=DF(I)
      MET=MET1+2
      ASSIGN 202 TO METKA
      GOTO 100
  202 DFF1=FF1-DDF(1)
      DFF2=FF2-DF(1)
      IF(ABS(DFF1)+ABS(DFF2)-1.E-4) 35,35,32
   32 DET=DDF(2)*DF(3)-DDF(3)*DF(2)
      DLA=DLA+(DFF1*DF(3)-DFF2*DDF(3))/DET
      AA=-(DFF1*DF(2)-DFF2*DDF(2))/DET
C=============================================
      IF(MET1.EQ.0) GOTO 33
      TEMP=TEMP+AA
      IF(TEMP-1.E-1) 36,36,34
   36 TEMP=1.E-1
      GOTO 34
   33 DEL=DEL+AA
   34 I3=34
C      PRINT  99,I3,TEMP,DLA, DEL,DDF,DF
      GOTO 30
   35 I3=35
C      PRINT 98,I3,TEMP,DLA,DEL,DFF1,DFF2
      GOTO(15,20,45),MET2
   40 DEL=0.
      MET=1
   98 FORMAT(I3,3X,3(F8.5,3X),2(E12.5,2X))
      ASSIGN 203 TO METKA
   41 GOTO100
  203 DFF1=FF1-DF(1)

      IF(ABS(DFF1)-1.E-4) 45,45,42

   42 DLA=DLA+DFF1/DF(2)
      I3=42
C      PRINT 99,I3,TEMP,DEL,DLA,DF,DDF
   99 FORMAT(/I5,3X,11(F6.3,1X))
      GOTO 41
   45 RES(10)=DLA
      RES(11)=DEL
      DO 46 I=1,5
   46 DDF(I)=0.
      MET=5
      ASSIGN 212 TO METKA
      GOTO 100
  212 DO 47 I=1,3
      RES(11+I)=DF(I)
      RES(14+I)=DDF(I)
   47 CONTINUE
      RES(18)=ABS(RES(14)*RES(17)-RES(16)**2)
      RES(19)=DDF(4)
      RES(12)=RES(12)-RES(7)-DEL**2/RES(2)
      RES(20)=PI**2*RES(15)/6./TEMP
      RES(21)=.5*RES(13)/TEMP
      RES(22)=RES(19)*(1.+.33333*BET)/(1.-.66667*BET)
      IF(TEMP-RES(5)) 50,55,55
   50 FUN=1.-(RES(11)/RES(4))**2
      RES(22)=.33333*RES(22)*(1.+2.*RES(5)/TEMP*FUN)
   55 RETURN
  100 DO 101 I=1,3
  101 DF(I)=0.
      I=0
  110 I=I+1
      EP=EPS(I)-DLA
      EKP=SQRT(EP**2+DEL**2)
      FJ=1.
      IF(EPS(I).EQ.EBL) FJ=FJ-1.
      FN=FERMI(EKP,TEMP)
      FN1=1.-2.*FN
      FN2=2.*FN*(1.-FN)
      FN3=FN1/EKP
      IF(TEMP)112,112,111
  111 FN3=FN3-FN2/TEMP
  112 CONTINUE
      GOTO(115,120,130,140,150), MET
  115 DF(1)=DF(1)+FJ*(1.-EP/EKP*FN1)
      AA=FJ*EP/EKP**2*FN3
      DF(2)=DF(2)+FJ/EKP*FN1-EP*AA
      DF(3)=DF(3)+DEL*AA
      GOTO 160
  120 DF(1)=DF(1)+FJ/EKP*FN1
      AA=FJ/EKP**2*FN3
      DF(2)=DF(2)+EP*AA
      DF(3)=DF(3)-DEL*AA
      GOTO 160
  130 DF(1)=DF(1)+FJ*(1.-EP/EKP*FN1)
      AA=FJ*FN2/TEMP
      DF(2)=DF(2)+AA
      DF(3)=DF(3)+AA*EP/TEMP
      GOTO 160
  140 DF(1)=DF(1)+FJ*FN1/EKP
      DF(2)=DF(2)+FJ*EP/EKP**2*FN3
      DF(3)=DF(3)-FJ*FN2/TEMP**2
      GOTO 160
  150 DF(1)=DF(1)+FJ*(EP+DLA)*(1.-EP/EKP*FN1)
      AK=FJ*FN2
      BK=FJ*(DEL/EKP*FN1)**2
      DF(3)=DF(3)+AK+BK
      IF(TEMP-1.E-3) 160,160,151
  151 DF(2)=DF(2)+2.*FJ*(EKP/TEMP*FN-ALOG(1.-FN))
      DDF(1)=DDF(1)+AK
      DDF(2)=DDF(2)-(EP+DLA)*AK-DLA*BK
      DDF(3)=DDF(3)+((EP+DLA)**2+DEL**2)*AK+DLA**2*BK
      DDF(4)=DDF(4)+IPS(I)**2*AK
  160 IF(I.LT.NLEV) GOTO 110
  161 GOTO METKA,(201,202,203,211,212)
      END

      FUNCTION FERMI(EK,T)
      IF(T-1.E-3) 3,3,1
    1 AA=EK/T
      IF(AA-30.) 2,3,3
    2 FERMI=1./(1.+EXP(AA))
      RETURN
    3 FERMI=0.
      RETURN
      END

      SUBROUTINE QVIBR(T,OM,A,LAM,Q)
C***** QVIBR WITH PHENOMENOLOGICAL DAMPING ***
C***** TAKEN FROM SYSTEMATICS A.IGNATYUK ET AL.CONF.MITO (1988)
      Q=1.
      IF(T.LT.0.05) GOTO 10
      CGAM=.0075*A**.33333
      GAM=CGAM*(OM**2+(2.*3.141593*T)**2)
      FN=EXP(-GAM/OM/2)/(EXP(OM/T)-1.)
      IF(FN.LT.0.) GOTO 10
      U=LAM*OM*FN
      S=LAM*((1.+FN)*ALOG(1.+FN)-FN*ALOG(FN))
      Q=EXP(S-U/T)
   10 RETURN
      END

      SUBROUTINE QROT(A,BET,SIG,U,QR)
C***** QROT WITH PHENOMENOLOGICAL DAMPING ***
C***** TAKEN FROM G.HANSEN, A.JENSEN. NUCL.PHYS.,A406(1983)236
      UCR=120.*BET*BET*A**.33333
      DCR=1400.*BET*BET/A**.66666
      IF(BET) 10,10,11
   10 QR=1
      GOTO 12
   11 FX=(U-UCR)/DCR
      IF(FX-30.) 9,10,10
    9 QR=(SIG-1.)/(1.+EXP(FX))+1.
   12 RETURN
      END

      Subroutine SPL(EPSZ,IPSZ,NLEVZ,EPSN,IPSN,NLEVN)
      character*12 Filename,Name
      character*80 Comment
      integer*4 nlvtmp
      PARAMETER(IDIM=300)
      DIMENSION EPSZ(IDIM),IPSZ(IDIM),EPSN(IDIM),IPSN(IDIM)
      REAL*4 GZ,GN,TMAX,TMIN,TSTEP,BET,Z,A,FTMP
      COMMON/TEMP/TMIN,TMAX,TSTEP,Z,A,GZ,GN
      COMMON BET

c     WRITE(*,*) 'Please enter the name of INPUT FILE:'
c     Read(*,'(A)') Filename

C     Open(10,File=FileName,Status='OLD',err=127)
       Read (5,'(A)')Name
       Read (5,'(A)')Comment
       READ (5,*) GZ,GN,TMAX,TMIN,TSTEP
C     CLOSE(10)

      Open(12,File=Name,Status='OLD',err=126)
C     Open(15,File='BCS.SPE')
      read(12,*)Z,AN,A,FTMP,BET
      DO 1000 Kind=1,2
       read(12,*)nlvtmp
C      write(15,*)nlvtmp
       IF(Kind.eq.1) then
        NLEVN=nlvtmp
       ELSE
        NLEVZ=nlvtmp
       ENDIF
       Do 110 I=1,nlvtmp
        IF(Kind.eq.1) then
          Read(12,*) EPSN(I),STMP,PAR
C         EPS = Eenergy, IPS = M-1/2
          IPSN(I)=PAR*STMP
C          write(*,*) Kind,EPSN(I),IPSN(I)
C          write(15,*) EPSN(I),IPSN(I)
        ELSE
          Read(12,*) EPSZ(I),STMP,PAR
C         EPS = Energy, IPS = M-1/2
          IPSZ(I)=PAR*STMP
C         write(*,*) Kind,EPSZ(I),IPSZ(I)
C         write(15,*) EPSN(I),IPSN(I)
        ENDIF
110    continue
1000  Continue
C     Close(15)
      Close(12)
      Return

 126  stop 'INPUT SPL FILE NOT FOUND'
 127  stop 'INPUT FILE NOT FOUND'
      End
