C
C                          microscopic.for v2.0
C     MICROSCOPICAL PARTICLE-HOLE STATE DENSITIES CALCULATION (May 1997)
C
C     Contact person: Dr.Roberto Capote Noy
C     e-mail: rcapote@us.es ; rcapotenoy@yahoo.com
C
C  This code is an improved version (almost completely rewritten)
C  of the original ICAR code written by M.HERMAN and G.REFFO (1986-1988).
C  It was developed by R.CAPOTE and R.PEDROSA in 1995-1996(BOLOGNA-HAVANA).
C
C  REFERENCES:
C  1) Herman M., Reffo G., Phys. Rev.C36(1987) 1546
C  2) Herman M., Reffo G.,    Comp.Phys.Comm.47(1987) 103.
C  3) Herman M., Reffo G., "Combinatorial Approach to few quasiparticle
C     state densities", Proc.Worshop on Applied Nuclear Theory and
C     Nuclear Model Calculations for Nuclear Technology Applications,
C     15 Feb.-19 Mar.1988, ICTP Trieste, Italia, Edited by M.K.Mechta
C     and J.J.Schmidt, World Scientific, Singapore 1989.
C  4) Herman M., Reffo G., Rego R.A., Phys.Rev.C37(1988)797
C  5) Capote R., Pedrosa R., DENSIDAD code(1996), unpublished.
C
C  COMMENT:
C  Using a set of single-particle levels(\single-particle-levels\ directory)
C  you will be able to calculate usually only STATE densities as the
C  corresponding single-particle level schemes are calculated for some
C  deformation. Therefore, using Moller single-particle levels it is
C  impossible to get level density RHO by the method employed in the code
C       (level density RHO(E,J)=OMEGA(M=J)-OMEGA(M=J+1))
C
C  R.Capote can provide single-particle levels at zero deformation for
C  spherical nucleus on request.
C
C  OPERATING ENVIROMENT: LINUX workstation
C
C  The code was developed and tested on PC computer running LINUX.
C  with the F77 compiler.
C
C--------------------------------------------------------------------------
C  INPUT INFORMATION:
C
C  It is designed for interactive input, which is self explained.
C  You should prepare also some input files as well as files, containing
C  shell model single-particle levels. The single-particle levels files
C  are provided by the code moller_levels.for(included in the Supplement)
C
C  For a given nucleus XXXX all corresponding input files of the type:
C   nXXXX11.inp,nXXXX22.inp,nXXXX33.inp  are used if present.
C   pXXXX11.inp,pXXXX22.inp,pXXXX33.inp
C   (XXXX=ni58 in the test example).
C--------------------------------------------------------------------------
C
C  The main difference with original code is related to the particle-hole
C  state densities at low excitation energies. The original code do not
C  give a same value of the w(p,h,E) for the spherical and sligthly deformed
C  nucleus. But, from the physical point of view, you should get the same
C  results in both cases with small redistribution of the levels.
C  Differences were related to different treatment of the excited particles
C  and holes in the subshell corresponding to the Fermi level.
C
C  The code was tested up to 3p-3h configurations, checking by hand that
C  all possible configurations were generated.
C-----------------------------------------------------------------------------
C  SAMPLE INPUT FILE: pfe5811.inp
Cline#1:  P                     Proton || Neutron DENSITY CALCULATIONS
Cline#2:  1,1                   Particle/Hole Numbers
Cline#3:  58,26                 A,Z of the nucleus
Cline#4:  100,0.D0              Number of SPL orbitals to be considered,Bn
Cline#5:  1,4.D0,1              Calc.Key(0,1,2) ,Eog(ESM spacing),DEBUG key(0/1)
Cline#6:  13.85D0               G*A
Cline#7:  20                    Maximum excitation energy to be considered
Cline#8:  z026a058.spe          Shell Model input spectrum filename
C-----------------------------------------------------------------------------
C  COMMENTS: 1) The shell model input spectra file is provided by
C      retrieve.for code, which extract single-particle levels from the
C      Moeller-Nix or HFBCS single-particle levels database
C            2) Bn = Binding energy is used only if you want to get bounded
C      p-h state densities(for MSC calc.for example)
C            3)  Calc.key is as follows:
C            0 : Equidistant Spacing Model with constant spacing Eog from the
C                input file is assumed
C            1 : SHELL MODEL SPECTRUM INPUT(obtained by retrieve.for)
C            2 : Equidistant Spacing Model with constant spacing Eog from the
C                SHELL MODEL SPECTRUM
C ----------------------------------------------------------------------------
C   Using this input file you will get the following output files(LD=level density):
C pfe5811.fer - ASCII file containing p-h LD groupped into 1 MeV (Fermi model)
C pfe5811.dat - ASCII file containing p-h LD groupped into 1 MeV bins(BCS model)
C pfe5811.nav - ASCII file containing p-h LD in 100 KeV bins(BCS model).
C pfe5811.p_h - Debug file. Contains all generated configurations
C               with energy less than 17 MeV(only present when DEBUG key=1).
C pfe5811.out - Detailed ASCII output file of the level density calculations.
C              Contains parity distribution, level and state density for each
C              parity, momentum distribution, brief information about BCS
C              solution, etc.
C
      PROGRAM DENSIDAD
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 DIM
      PARAMETER(DIM = 500)
      INTEGER*4 JA, JP, NLV, NP, NH, KINd, CALc, OLV, MAXener
      LOGICAL DEFor, DEBug
      REAL*8 GI, BE, EOG, ELV, SLV, ENGrgas, ENGrbcs, SS
      CHARACTER*7 FILename
      CHARACTER*4 root
      INTEGER*4 NEN, NER, IERm, i
      REAL*8 SFN, SFEr
      COMMON /ERR   / NEN, NER, SFN, SFEr, IERm(25)
      COMMON /FILE  / FILename
      COMMON /GROUND/ ENGrgas, ENGrbcs, DEBug
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
      WRITE(*, 99001)
99001 FORMAT('  Nucleus(4 lett.:ni58,s32n,pu40) => ')
      READ(*, '(A)')root
C
      DO i = 1, 6
         IF(i.EQ.1)THEN
            FILename = 'n'//root(1:4)//'11'
         ELSEIF(i.EQ.2)THEN
            FILename = 'p'//root(1:4)//'11'
         ELSEIF(i.EQ.3)THEN
            FILename = 'n'//root(1:4)//'22'
         ELSEIF(i.EQ.4)THEN
            FILename = 'p'//root(1:4)//'22'
         ELSEIF(i.EQ.5)THEN
            FILename = 'n'//root(1:4)//'33'
         ELSEIF(i.EQ.6)THEN
            FILename = 'p'//root(1:4)//'33'
         ENDIF
C
         OPEN(1, FILE = FILename//'.inp', STATUS = 'OLD', ERR = 100)
         OPEN(3, FILE = FILename//'.out')
C
 50      CALL SPL
C
         IF(NP.GT.3 .OR. NH.GT.3)STOP 'MAX(P,H)>3'
C
         NEN = 0
         NER = 0
         SFEr = 0.
         SFN = 0.
         CALL PARTHOLE
         CALL OUTPUT
         IF(NP + NH.EQ.0 .OR. MOD(NP + NH, 2).NE.MOD(JP, 2))GOTO 50
 100     CLOSE(1)
         CLOSE(3)
      ENDDO
      END
C
      SUBROUTINE CONFIG
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 DIM
      PARAMETER(DIM = 500)
C
C   Controls configuration generation, BCS solution and energy
C   determination; calculates parity and spin projection of the
C   states. Only one exciton configuration are considered.
C
      INTEGER*4 i, j, ien, ienn, k, kq, js, ne, NCOnfig
      INTEGER*4 JA, JP, NLV, NP, NH, KINd, CALc, fermi, itmp(10)
      INTEGER*4 OLV, lvshole(5), lvspart(5), MAXener, ma, newpp, newhh
      INTEGER*4 parity, old, new, oldp, oldh, newp, newh, ips, SHM, 
     &          DENbcs, DENgas
      REAL*8 GI, BE, EOG, ELV, SLV, EMAx, EHM, chi, ENERGY, epp, ehh
      REAL*8 egas, pmp(10), gap, lamda, enstate, psh, sj, enold
      LOGICAL start, DEFor, impar, DEBug
      REAL*8 ENGrgas, ENGrbcs
      INTEGER*4 NEN, NER, IERm
      REAL*8 SFN, SFEr, SS
      COMMON /ERR   / NEN, NER, SFN, SFEr, IERm(25)
      COMMON /CANT_CONF/ NCOnfig
      COMMON /GROUND/ ENGrgas, ENGrbcs, DEBug
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
      COMMON /LIMIT / EMAx, EHM, SHM(101), DENbcs(400, 30, 2), 
     &                DENgas(400, 30, 2)
      DATA start/.TRUE./
C
      impar = .FALSE.
      IF(MOD(JP, 2).EQ.1)impar = .TRUE.
      fermi = JP/2 + MOD(JP, 2)
C
      DO k = 1, 5
         lvspart(k) = 0
         lvshole(k) = 0
      ENDDO
C
      DO i = 1, NP
         OLV(fermi + i) = 1
         lvspart(i) = i
      ENDDO
C
      DO i = 1, NH
         OLV(fermi - i + 1) = 1
         lvshole(i) = i
      ENDDO
C
      newhh = -fermi
      ehh = ELV(fermi)
C
C-----GENERATION OF PARTICLE CONFIGURATION--------------------------
C
 100  IF(NP.NE.0)THEN
         CALL PNXCB(lvspart, NP, NLV - fermi + 1, oldp, newp)
         old = oldp + fermi - 1
         new = newp + fermi - 1
         newpp = new
         epp = ELV(newpp)
         OLV(old) = 0
         OLV(new) = 1
C        You can not fill FERMI level for EVEN NUCLEUS
         IF(.NOT.impar .AND. new.EQ.fermi)GOTO 300
      ENDIF
C
C-----GENERATION OF HOLE CONFIGURATION------------------------------
C
 200  IF(NH.NE.0)THEN
         CALL PNXCB(lvshole, NH, fermi, oldh, newh)
         old = fermi - oldh + 1
         new = fermi - newh + 1
         newhh = -new
         ehh = ELV(new)
         OLV(old) = 0
         OLV(new) = 1
         IF(impar .AND. new.EQ.fermi)GOTO 300
      ENDIF
C
      j = 0
      DO i = 1, NLV
         IF(OLV(i).EQ.1)j = j + 1
      ENDDO
C
      IF(impar)THEN
         IF(j.EQ.NP + NH - 1 .AND. OLV(fermi).EQ.1)GOTO 300
         IF(j.EQ.NP + NH - 1)THEN
            OLV(fermi) = 1
            j = j + 1
         ENDIF
      ENDIF
C
      IF(j.EQ.(NP + NH))THEN
C
C        Excitation energy calculation
         egas = ENERGY(0.D0, 1.D0) - ENGrgas
C        MAXIMUM ENERGY CUT-OFF
         IF(egas.LE.DBLE(MAXener))THEN
C
C
C-------------------------------------------------------------------------------------
C           ONLY FOR DEBUG PRINTING
            IF(DEBug)THEN
               j = 0
               DO i = 1, NLV
                  IF(OLV(i).EQ.1)THEN
                     j = j + 1
                     IF(impar)THEN
                        IF(i.EQ.fermi)itmp(j) = i
                     ELSE
                        IF(i.EQ.fermi)itmp(j) = -i
                     ENDIF
                     IF(i.LT.fermi)itmp(j) = -i
                     IF(i.GT.fermi)itmp(j) = i
                  ENDIF
               ENDDO
            ENDIF
C
C----------------------------------------------------
C----------Solution of BCS equations----------------
            IF(start .OR. (ABS(egas-enold).GT.0.005))THEN
               start = .FALSE.
               enold = egas
               CALL CLAMDA(gap, lamda, chi)
               enstate = ENERGY(gap, lamda) - ENGrbcs
               psh = enstate - egas
               ips = psh*10.D0 + 50.D0
               IF(ips.LT.1 .OR. ips.GT.100)ips = 101
            ENDIF
C
C-------------------------------------------------------------------------------------
C           ONLY FOR DEBUG PRINTING
            IF(DEBug .AND. enstate.LE.17)THEN
               WRITE(10, 99001)(itmp(i), i = 1, j)
99001          FORMAT(/1X, 'Particles & Holes:', 10(I3, 2X))
               WRITE(10, 99002)newhh, ehh, newpp, epp, egas, gap, lamda, 
     &                         chi, enstate
99002          FORMAT(1X, '[', I3, ':', F6.2, ',', I2, ':', F6.2, ']', 
     &                '=> U(Gas)=', F6.2, ' D=', F5.3, ' L=', F6.2, 
     &                ' CHI=', G8.2, ' U(BCS)=', F6.2)
            ENDIF
C----------------------------------------------------
            IF(enstate.GT.0.)NCOnfig = NCOnfig + 1
C------Determination of parity for the configuration---------
            parity = 1
            DO i = 1, NP
               ma = lvspart(i) + fermi - 1
               parity = parity*SIGN(1.D0, SLV(ma))
               pmp(i) = ABS(SLV(ma))
            ENDDO
            DO i = 1, NH
               ma = fermi + 1 - lvshole(i)
               parity = parity*SIGN(1.D0, SLV(ma))
               pmp(NP + i) = ABS(SLV(ma))
            ENDDO
            IF(parity.EQ. - 1)parity = 2
C
C------Coupling of all spin projections to generate nuclear states----
            ne = NP + NH
            DO i = 1, 2**(ne - 1)
               kq = i - 1
               sj = 0.D0
               DO j = 1, ne
                  sj = sj + (1 - 2*MOD(kq, 2))*pmp(j)
                  kq = kq/2
               ENDDO
               js = ABS(sj) + 1
               IF(js.LE.30)THEN
C
C                 STORING OF THE NUCLEAR STATE IN THE APROPRIATE ELEMENT OF DMN
C
                  IF(enstate.GT.0.)THEN
                     NCOnfig = NCOnfig + 1
                     ien = enstate*10 + 1
                     DENbcs(ien, js, parity) = DENbcs(ien, js, parity)
     &                  + 2
                     SHM(ips) = SHM(ips) + 2.D0
                     EHM = EHM + 2.D0*psh
                  ENDIF
C
                  IF(egas.GT.0.)THEN
                     ienn = egas*10 + 1
                     DENgas(ienn, js, parity) = DENgas(ienn, js, parity)
     &                  + 2
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
      ENDIF
C
 300  IF(NH.NE.0)THEN
         IF(lvshole(NH).NE.NH)GOTO 200
      ENDIF
      OLV(fermi) = 0
      IF(NP.NE.0)THEN
         IF(lvspart(NP).NE.NP)GOTO 100
      ENDIF
C
      DO i = 1, NLV
         OLV(i) = 0
      ENDDO
C
      RETURN
      END
C
      SUBROUTINE PROMOTED
C
C   Controls configuration generation, BCS solution and energy
C   determination; calculates parity and spin projection of the
C   states. Only excited pair configurations are considered.
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 DIM
      PARAMETER(DIM = 500)
      INTEGER*4 ipart, ihole, i, ien, ienn
      INTEGER*4 JA, JP, NLV, NP, NH, KINd, CALc, fermi, MAXener
      INTEGER*4 OLV, parhole(2), parpart(2), newpp, newhh, NCOnfig
      INTEGER*4 old, new, oldp, oldh, newp, newh, ips, SHM, DENbcs, 
     &          DENgas
      LOGICAL DEFor, start, impar, DEBug
      REAL*8 GI, BE, EOG, ELV, SLV, EMAx, EHM, chi, ENERGY, enold
      REAL*8 egas, gap, lamda, enstate, psh, epp, ehh
      REAL*8 ENGrgas, ENGrbcs
      INTEGER*4 NEN, NER, IERm
      REAL*8 SFN, SFEr, SS
      COMMON /ERR   / NEN, NER, SFN, SFEr, IERm(25)
      COMMON /CANT_CONF/ NCOnfig
      COMMON /GROUND/ ENGrgas, ENGrbcs, DEBug
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
      COMMON /LIMIT / EMAx, EHM, SHM(101), DENbcs(400, 30, 2), 
     &                DENgas(400, 30, 2)
C
      impar = .FALSE.
      fermi = JP/2 + MOD(JP, 2)
      IF(MOD(JP, 2).EQ.1)impar = .TRUE.
C
      DO ipart = 0, NP/2
         DO ihole = 0, NH/2
C
            DO i = 1, 2
               parpart(i) = 0
               parhole(i) = 0
            ENDDO
C
            DO i = 1, NLV
               OLV(i) = 0
            ENDDO
C
            start = .TRUE.
C
            IF(ipart.EQ.0 .AND. ihole.EQ.0)GOTO 100
C
            DO i = 1, ipart
               OLV(fermi + i) = 2
               parpart(i) = i
            ENDDO
C
            DO i = 1, ihole
               OLV(fermi - i + 1) = -2
               parhole(i) = i
            ENDDO
C
            newpp = 0
            newhh = 0
C
C---------GENERATION OF PAIR CONFIGURATION (PARTICLES)
 20         IF(ipart.NE.0)THEN
               CALL PNXCB(parpart, ipart, NLV - fermi + 1, oldp, newp)
               old = oldp + fermi - 1
               new = newp + fermi - 1
               newpp = new
               epp = ELV(newpp)
               IF(old.GT.fermi)OLV(old) = 0
C              You can not fill FERMI level(EVEN)
               IF(.NOT.impar .AND. new.EQ.fermi)GOTO 60
               OLV(new) = 2
C
               egas = ENERGY(0.D0, 1.D0) - ENGrgas
               IF(egas.GT.DBLE(MAXener))GOTO 60
C
            ENDIF
C
C---------GENERATION OF PAIR CONFIGURATION (HOLES)
 40         IF(ihole.NE.0)THEN
               CALL PNXCB(parhole, ihole, fermi, oldh, newh)
               old = fermi - oldh + 1
               new = fermi - newh + 1
               newhh = -new
               ehh = ELV(new)
               IF(impar)THEN
                  IF(OLV(old).EQ. - 2)OLV(old) = 0
               ELSE
                  OLV(old) = 0
               ENDIF
               OLV(new) = -2
            ENDIF
C
            IF(NP.GT.2*ipart .OR. NH.GT.2*ihole)THEN
C---------GENERATION OF CONFIGURATIONS FOR EXCITONS REMAINING AFTER
C-------------------EXCLUSION OF THE PROMOTED PAIRS
               CALL CONFIG_PAR(ipart, ihole, parpart, parhole)
            ELSE
C----------SOLUTION OF BCS EQUATIONS FOR PURE PROMOTED PAIR STATES
               egas = ENERGY(0.D0, 1.D0) - ENGrgas
               IF(egas.LE.DBLE(MAXener) .AND. egas.GE.0.005D0)THEN
C
                  IF(start .OR. (ABS(egas-enold).GT.0.005))THEN
                     start = .FALSE.
                     enold = egas
                     CALL CLAMDA(gap, lamda, chi)
                     enstate = ENERGY(gap, lamda) - ENGrbcs
                     psh = enstate - egas
                     ips = psh*10.D0 + 50.D0
                     IF(ips.LT.1 .OR. ips.GT.100)ips = 101
                  ENDIF
C
C-------------------------------------------------------------------------------------
C                 ONLY FOR DEBUG PRINTING
                  IF(DEBug .AND. ipart + ihole.LE.6 .AND. 
     &               enstate.LT.17.00)THEN
                     DO i = 1, NLV
                        IF(OLV(i).EQ. - 2)WRITE(10, 99001) - i
99001                   FORMAT(/1X, 'Hole     Pair:', I3)
                        IF(OLV(i).EQ.2)WRITE(10, 99002)i
99002                   FORMAT(1X, 'Particle Pair:', I3)
                     ENDDO
                     WRITE(10, 99003)egas, gap, lamda, chi, enstate
99003                FORMAT(1X, 'U(Gas)=', F6.2, ' D=', F5.3, ' L=', 
     &                      F6.2, ' CHI=', G8.2, ' U(BCS)=', F6.2)
                  ENDIF
C
C
                  IF(enstate.GT.0.)THEN
                     NCOnfig = NCOnfig + 1
                     ien = enstate*10 + 1
                     DENbcs(ien, 1, 1) = DENbcs(ien, 1, 1) + 1
                     SHM(ips) = SHM(ips) + 1.D0
                     EHM = EHM + psh
                  ENDIF
C
                  IF(egas.GT.0.)THEN
                     ienn = egas*10 + 1
                     DENgas(ienn, 1, 1) = DENgas(ienn, 1, 1) + 1
                  ENDIF
               ENDIF
C
            ENDIF
 60         IF(ihole.NE.0)THEN
               IF(parhole(ihole).NE.ihole)GOTO 40
            ENDIF
            IF(ipart.NE.0)THEN
               IF(parpart(ipart).NE.ipart)GOTO 20
            ENDIF
 100     ENDDO
      ENDDO
      RETURN
      END
C
      SUBROUTINE CONFIG_PAR(Ipart, Ihole, Parpart, Parhole)
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 DIM
      PARAMETER(DIM = 500)
C
C   Controls configuration generation, BCS solution and energy
C   determination; calculates parity and spin projection of the
C   states. Configuration with two exciton in one and another in the second orbital
C   are considered.
C
      INTEGER*4 i, j, k, ien, ienn, kq, js, ne, npares, jpp, jh, ip, ih
      INTEGER*4 JA, JP, NLV, NP, NH, KINd, CALc, fermi
      INTEGER*4 OLV, lvshole(5), lvspart(5), MAXener, itmp(10), itmpp(4)
      INTEGER*4 Ihole, Ipart, Parpart(2), Parhole(2), ma, NCOnfig
      INTEGER*4 parity, old, new, oldp, oldh, newp, newh, ips, SHM, 
     &          DENbcs, DENgas
      REAL*8 GI, BE, EOG, ELV, SLV, EMAx, EHM, chi, ENERGY, enold
      REAL*8 egas, pmp(10), gap, lamda, enstate, psh, sj
      LOGICAL start, DEFor, impar, DEBug
      REAL*8 ENGrgas, ENGrbcs
      INTEGER*4 NEN, NER, IERm
      REAL*8 SFN, SFEr, SS
      COMMON /ERR   / NEN, NER, SFN, SFEr, IERm(25)
      COMMON /CANT_CONF/ NCOnfig
      COMMON /GROUND/ ENGrgas, ENGrbcs, DEBug
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
      COMMON /LIMIT / EMAx, EHM, SHM(101), DENbcs(400, 30, 2), 
     &                DENgas(400, 30, 2)
C
      start = .TRUE.
      impar = .FALSE.
      fermi = JP/2 + MOD(JP, 2)
      IF(MOD(JP, 2).EQ.1)impar = .TRUE.
C
      DO k = 1, 5
         lvspart(k) = 0
         lvshole(k) = 0
      ENDDO
C
      DO k = 1, 2
         IF(Parpart(k).EQ.0)Parpart(k) = 1000
         IF(Parhole(k).EQ.0)Parhole(k) = 1000
      ENDDO
C
C-------------------------------------------------------------------------------------
C     ONLY FOR DEBUG PRINTING
      IF(DEBug)THEN
         i = 0
         DO ien = 1, 2
            IF(Parpart(ien).LT.1000)THEN
               j = (Parpart(ien) + fermi - 1)
               i = i + 1
               itmpp(i) = j
            ENDIF
            IF(Parhole(ien).LT.1000)THEN
               k = (fermi - Parhole(ien) + 1)
               i = i + 1
               itmpp(i) = -k
            ENDIF
         ENDDO
         npares = Ipart + Ihole
      ENDIF
C
      j = 0
      DO i = 1, NP - 2*Ipart
         j = j + 1
 50      IF(ABS(OLV(fermi+j)).EQ.2)THEN
            j = j + 1
            GOTO 50
         ENDIF
         OLV(fermi + j) = 1
         lvspart(i) = i
      ENDDO
C
      j = 0
      DO i = 1, NH - 2*Ihole
         j = j + 1
 100     IF(ABS(OLV(fermi-j+1)).EQ.2)THEN
            j = j + 1
            GOTO 100
         ENDIF
         OLV(fermi - j + 1) = 1
         lvshole(i) = i
      ENDDO
C-----GENERATION OF PARTICLE CONFIGURATION----------------
 200  IF(NP.NE.2*Ipart)THEN
         CALL PNXCB(lvspart, NP - 2*Ipart, NLV - fermi + 1 - Ipart, 
     &              oldp, newp)
         old = oldp + fermi - 1
         new = newp + fermi - 1
C
         DO i = 1, Ipart
            IF(new.GE.Parpart(i) + fermi - 1)new = new + 1
            IF(old.GE.Parpart(i) + fermi - 1)old = old + 1
            IF(new.GT.NLV .OR. old.GT.NLV)GOTO 400
         ENDDO
C
         IF(impar)THEN
            IF(OLV(old).EQ.1)OLV(old) = 0
C           If(Old.Ge.Fermi)Olv(Old)=0
         ELSE
            IF(old.GT.fermi)OLV(old) = 0
C           You can not fill FERMI level(EVEN)
            IF(new.EQ.fermi)GOTO 400
         ENDIF
         OLV(new) = 1
C
      ENDIF
C
C-----GENERATION OF HOLE CONFIGURATION------------------
 300  IF(NH.NE.2*Ihole)THEN
         CALL PNXCB(lvshole, NH - 2*Ihole, fermi - Ihole, oldh, newh)
         old = fermi - oldh + 1
         new = fermi - newh + 1
         DO i = 1, Ihole
            IF(new.LE.fermi - Parhole(i) + 1)new = new - 1
            IF(old.LE.fermi - Parhole(i) + 1)old = old - 1
            IF(new.LT.1 .OR. old.LT.1)GOTO 400
         ENDDO
C
         IF(impar)THEN
            IF(OLV(old).EQ.1)OLV(old) = 0
C           You can not get particle from the FERMI level(ODD)
            IF(new.EQ.fermi)GOTO 400
         ELSE
            IF(old.LE.fermi)OLV(old) = 0
         ENDIF
C
         OLV(new) = 1
C
      ENDIF
C
      jh = 0
      jpp = 0
      ip = 0
      ih = 0
      IF(impar)THEN
         DO i = 1, fermi - 1
            IF(OLV(i).EQ. - 2)ih = ih + 1
            IF(OLV(i).EQ.1)jh = jh + 1
         ENDDO
C
         IF(OLV(fermi).EQ. - 2)ih = ih + 1
C
         DO i = fermi, NLV
            IF(OLV(i).EQ.1)jpp = jpp + 1
            IF(OLV(i).EQ.2)ip = ip + 1
         ENDDO
      ELSE
         DO i = 1, fermi
            IF(OLV(i).EQ. - 2)ih = ih + 1
            IF(OLV(i).EQ.1)jh = jh + 1
         ENDDO
         DO i = fermi + 1, NLV
            IF(OLV(i).EQ.2)ip = ip + 1
            IF(OLV(i).EQ.1)jpp = jpp + 1
         ENDDO
      ENDIF
C
      IF(impar .AND. (2*ip + jpp.EQ.NP - 1))THEN
         IF(OLV(fermi).EQ.1)GOTO 400
         OLV(fermi) = 1
         jpp = jpp + 1
      ENDIF
C
      IF(jpp + 2*ip.EQ.NP .AND. jh + 2*ih.EQ.NH)THEN
C
         egas = ENERGY(0.D0, 1.D0) - ENGrgas
         IF(egas.LE.DBLE(MAXener))THEN
C-------------------------------------------------------------------------------------
C           ONLY FOR DEBUG PRINTING
            IF(DEBug)THEN
               j = 0
               DO ien = 1, NLV
                  IF(OLV(ien).EQ.1)THEN
                     j = j + 1
                     IF(impar)THEN
                        IF(ien.EQ.fermi)itmp(j) = ien
                     ELSE
                        IF(ien.EQ.fermi)itmp(j) = -ien
                     ENDIF
                     IF(ien.LT.fermi)itmp(j) = -ien
                     IF(ien.GT.fermi)itmp(j) = ien
                  ENDIF
               ENDDO
            ENDIF
C
C----------Solution of BCS equations----------------
            IF(start .OR. (ABS(egas-enold).GT.0.005))THEN
               start = .FALSE.
               enold = egas
               CALL CLAMDA(gap, lamda, chi)
               enstate = ENERGY(gap, lamda) - ENGrbcs
               psh = enstate - egas
               ips = psh*10.0 + 50
               IF(ips.LT.1 .OR. ips.GT.100)ips = 101
            ENDIF
C
C-------------------------------------------------------------------------------------
C           ONLY FOR DEBUG PRINTING
            IF(DEBug .AND. enstate.LT.17)THEN
               IF(npares.GT.0)WRITE(10, 99001)
     &                              (itmpp(k), ELV(ABS(itmpp(k))), 
     &                              k = 1, npares)
99001          FORMAT(/1X, 'PAIRS:', 4('[', I3, ':', F6.2, '] '))
               IF(j.GT.0)WRITE(10, 99002)(itmp(i), i = 1, j)
99002          FORMAT(1X, 'Particles & Holes:', 10(I3, 2X))
               WRITE(10, *)'IP=', ip, ' IH=', ih, ' JPP=', jpp, ' JH=', 
     &                     jh
               WRITE(10, 99003)egas, gap, lamda, chi, enstate
99003          FORMAT(1X, ' U(Gas)=', F6.2, ' D=', F5.3, ' L=', F6.2, 
     &                ' CHI=', G8.2, ' U(BCS)=', F6.2)
            ENDIF
C
            IF(enstate.GT.0.)NCOnfig = NCOnfig + 1
C------Determination of parity for the configuration---------
            parity = 1
            DO i = 1, NP - 2*Ipart
               ma = lvspart(i) + fermi - 1
               IF(lvspart(i).GE.Parpart(1))ma = ma + 1
               IF(lvspart(i).GE.Parpart(2))ma = ma + 1
               parity = parity*SIGN(1.D0, SLV(ma))
               pmp(i) = ABS(SLV(ma))
            ENDDO
            DO i = 1, NH - 2*Ihole
               ma = fermi + 1 - lvshole(i)
               IF(lvshole(i).GE.Parhole(1))ma = ma - 1
               IF(lvshole(i).GE.Parhole(2))ma = ma - 1
               parity = parity*SIGN(1.D0, SLV(ma))
               pmp(NP - 2*Ipart + i) = ABS(SLV(ma))
            ENDDO
            IF(parity.EQ. - 1)parity = 2
C
C-----Coupling of all spin projections to generate nuclear states----
            ne = NP + NH - 2*(Ipart + Ihole)
            DO i = 1, 2**(ne - 1)
               kq = i - 1
               sj = 0.D0
               DO j = 1, ne
                  sj = sj + (1 - 2*MOD(kq, 2))*pmp(j)
                  kq = kq/2
               ENDDO
               js = ABS(sj) + 1
               IF(js.LE.30)THEN
                  IF(enstate.GT.0.)THEN
                     ien = enstate*10 + 1
                     SHM(ips) = SHM(ips) + 2.D0
                     EHM = EHM + 2.D0*psh
                     DENbcs(ien, js, parity) = DENbcs(ien, js, parity)
     &                  + 2
                  ENDIF
                  IF(egas.GT.0.)THEN
                     ienn = egas*10 + 1
                     DENgas(ienn, js, parity) = DENgas(ienn, js, parity)
     &                  + 2
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
      ENDIF
C
 400  IF(NH.NE.2*Ihole)THEN
         IF(lvshole(NH - 2*Ihole).NE.NH - 2*Ihole)GOTO 300
      ENDIF
      IF(OLV(fermi).EQ.1)OLV(fermi) = 0
      IF(NP.NE.2*Ipart)THEN
         IF(lvspart(NP - 2*Ipart).NE.NP - 2*Ipart)GOTO 200
      ENDIF
C
      DO i = 1, NLV
         IF(OLV(i).EQ.1)OLV(i) = 0
      ENDDO
C
      RETURN
      END
C
      SUBROUTINE PARTHOLE
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 DIM
      PARAMETER(DIM = 500)
      CHARACTER*7 text(2)
      INTEGER*4 i, j, NCOnfig
      INTEGER*4 OLV, SHM, DENbcs, DENgas, MAXener
      INTEGER*4 JA, JP, NLV, NP, NH, KINd, CALc, cuasip, num
      LOGICAL DEFor, impar, DEBug
      CHARACTER*7 FILename
      REAL*8 GI, BE, EOG, EHM, chi
      REAL*8 ELV, SLV, gap, lamda, ENGrbcs, EMAx, egapgas, egapbcs
      REAL*8 ENGrgas, ENERGY
      INTEGER*4 NEN, NER, IERm, fermi
      REAL*8 SFN, SFEr, SS
      COMMON /FILE  / FILename
      COMMON /ERR   / NEN, NER, SFN, SFEr, IERm(25)
      COMMON /CANT_CONF/ NCOnfig
      COMMON /GROUND/ ENGrgas, ENGrbcs, DEBug
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
      COMMON /LIMIT / EMAx, EHM, SHM(101), DENbcs(400, 30, 2), 
     &                DENgas(400, 30, 2)
      DATA text/'NEUTRON', 'PROTON '/, impar/.FALSE./
C
      cuasip = NP*10 + NH
      fermi = JP/2 + MOD(JP, 2)
      IF(MOD(JP, 2).EQ.1)impar = .TRUE.
C
      WRITE(3, 99001)text(KINd)
99001 FORMAT('  Solution for ', A7, ' System')
C
      DO i = 1, 400
         DO j = 1, 30
            DENbcs(i, j, 1) = 0
            DENbcs(i, j, 2) = 0
            DENgas(i, j, 1) = 0
            DENgas(i, j, 2) = 0
         ENDDO
      ENDDO
C
      NCOnfig = 0
C
      EHM = 0.D0
      DO i = 1, 101
         SHM(i) = 0
      ENDDO
C
      DO i = 1, NLV
         OLV(i) = 0
      ENDDO
      IF(impar)OLV(fermi) = 1
C
C---------BCS Solution for the Ground State----------------------
      CALL CLAMDA(gap, lamda, chi)
      ENGrbcs = ENERGY(gap, lamda)
      ENGrgas = ENERGY(0.D0, 1.D0)
      WRITE(3, 99002)ENGrgas, gap, lamda, chi, ENGrbcs, 
     &               ENGrgas - ENGrbcs
99002 FORMAT(/'-------------Solution for BCS Ground State----------', 
     &       /'  Energy of Free Gas Ground State = ', F12.5, ' MeV', 
     &       /'  Gap= ', F10.5, '  Lamda= ', F10.5, ' MeV', '  CHI= ', 
     &       E10.3/, '    Energy of BCS Ground State = ', F12.5, ' MeV', 
     &       /'           Condensation Energy = ', F10.5, ' MeV')
      IF(cuasip.EQ.0)RETURN
      IF(MOD(NP + NH, 2).NE.MOD(JP, 2))THEN
         WRITE(3, 99003)
99003    FORMAT('   Only an IDIOT can look for odd configurations '/
     &          '        in even system')
         RETURN
      ENDIF
C
C----Calculation of the first excited state (without promoted pairs)----
      IF(.NOT.(impar))THEN
C
         num = NP
 50      IF(num.GT.0)THEN
            j = 1
 60         IF(ELV(fermi + j).EQ.ELV(fermi))THEN
               j = j + 1
               GOTO 60
            ENDIF
C
 80         IF(OLV(fermi + j).EQ.1)THEN
               j = j + 1
               GOTO 80
            ENDIF
            OLV(fermi + j) = 1
            num = num - 1
            GOTO 50
         ENDIF
         DO i = 1, NH
            OLV(fermi - i + 1) = 1
         ENDDO
C
      ELSEIF(NP.GT.1)THEN
C
         num = NP - 1
 100     IF(num.GT.0)THEN
            j = 1
 120        IF(ELV(fermi + j).EQ.ELV(fermi))THEN
               j = j + 1
               GOTO 120
            ENDIF
 140        IF(OLV(fermi + j).EQ.1)THEN
               j = j + 1
               GOTO 140
            ENDIF
            OLV(fermi + j) = 1
            num = num - 1
            GOTO 100
         ENDIF
C
         DO i = 1, NH
            OLV(fermi - i) = 1
         ENDDO
C
      ELSE
C
         j = 1
 150     IF(ELV(fermi + j).EQ.ELV(fermi))THEN
            j = j + 1
            GOTO 150
         ENDIF
C
         OLV(fermi) = 0
         OLV(fermi + j) = 1
C
      ENDIF
C
C
C     BCS solution to calculate Gap and Lamda
C
      CALL CLAMDA(gap, lamda, chi)
C
C
C
      egapbcs = ENERGY(gap, lamda) - ENGrbcs
      egapgas = ENERGY(0.D0, 1.D0) - ENGrgas
      WRITE(3, 99004)egapgas, gap, lamda, chi, egapbcs, 
     &               egapbcs - egapgas
99004 FORMAT(/'-----------Solution for First excited state----------', 
     &       /'   First Excited Energy of Free Gas'/
     &       '     (without promoted pairs)=', F12.5, ' MeV', 
     &       /'  Gap= ', F10.5, '  Lamda= ', F10.5, ' MeV', '  CHI= ', 
     &       E10.3/, '   First Excited Energy with Pairing Force'/
     &       '     (without promoted pairs)=', F12.5, ' MeV', 
     &       /'             Pairing Gap=', F10.5, ' MeV')
C
C-----Generation of the configurations and densities calculation-------
      DO i = 1, NLV
         OLV(i) = 0
      ENDDO
C
      NEN = 0
      NCOnfig = 0
C-------------------------------------------------------------------------------------
C     ONLY FOR DEBUG PRINTING
      IF(DEBug)OPEN(10, FILE = FILename//'.p_h')
      IF(DEBug)WRITE(10, 99005)
99005 FORMAT(1x, 
     &       ' Here is an enumeration of all generated configurations'/
     &      '  U(Gas) is the configuration excitation energy in the FGM'
     &      /
     &     '  U(BCS) is the configuration excitation energy in the BCSM'
     &     /'  D and L are DELTA and LAMBDA of the BCS solution'/)
      CALL CONFIG()
      CALL PROMOTED
      IF(DEBug)CLOSE(10)
C
      RETURN
      END
C
C
      SUBROUTINE CLAMDA(Gap, Lamda, Chi)
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 DIM, ITERMAX, ni, nf, ncalc
      REAL*8 EPS
      PARAMETER(DIM = 500, ITERMAX = 400, EPS = 1.D-7)
      INTEGER*4 JA, JP, NLV, NP, NH, KINd, CALc
      INTEGER*4 OLV, i, MAXener
      INTEGER*4 iter, fermi, num
      LOGICAL DEFor
      REAL*8 GI, BE, EOG, ELV, SLV, Gap, Lamda, gap2, lamdai, lambda
      REAL*8 dene, ex, Chi, a1, a2, b1, b2, c1, c2, efermi
      REAL*8 dellam, delgap, g, rele, epsj, det, dllast, delmax
      INTEGER*4 NEN, NER, IERm, ier
      REAL*8 SFN, SFEr, SS
      COMMON /ERR   / NEN, NER, SFN, SFEr, IERm(25)
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
C
      NEN = NEN + 1
      Gap = 0.D0
C
      fermi = JP/2 + MOD(JP, 2)
 100  IF(ELV(fermi).EQ.ELV(fermi + 1))THEN
         fermi = fermi + 1
         GOTO 100
      ENDIF
C
      efermi = ELV(fermi)
C
      ni = 1
      nf = NLV
C
      ncalc = nf - ni + 1
C
C------ Initial Guess ------------
      lambda = fermi
      lamdai = 0.5D0*(ELV(fermi) + ELV(fermi + 1))
C
      IF(GI.EQ.0.D0)RETURN
C
      num = 0
C
      DO i = ni, nf
         IF(OLV(i).EQ.1)num = num + 1
         IF(OLV(i).EQ.2)num = num + 2
      ENDDO
C
      dene = (ELV(nf) - ELV(ni))/ncalc
      delmax = 2.D0*dene
      g = GI/DBLE(JA)
      Chi = 0.D0
      iter = 0
      c1 = 0
      c2 = (g/2.D0*ncalc)**2
      DO iter = 1, 30
         gap2 = (c1 + c2)/2.D0
         IF(DABS(c1 - c2).LE.0.1D0)GOTO 200
         b1 = -2.D0/g
         DO i = ni, nf
            IF(OLV(i).EQ.0)THEN
               rele = ELV(i) - lamdai
               epsj = SQRT(gap2 + rele*rele)
               IF(epsj.LT.0.005)GOTO 150
               b1 = b1 + 1.D0/epsj
            ENDIF
         ENDDO
 150     IF(b1.LE.0.D0)THEN
            c2 = gap2
         ELSE
            c1 = gap2
         ENDIF
      ENDDO
C
      ex = EXP(dene/g)
      gap2 = SQRT(DBLE(JP*(2*ncalc-JP)))*ex/(ex**2 - 1.)*dene
      gap2 = gap2*gap2
 200  dllast = delmax
 300  iter = iter + 1
      a1 = 0.D0
      a2 = 0.D0
      b1 = 2.D0/g
      b2 = DBLE(JP - num)
      DO i = ni, nf
         IF(OLV(i).EQ.0)THEN
            rele = ELV(i) - lamdai
            epsj = SQRT(gap2 + rele*rele)
C
            IF(epsj.LT.0.005)GOTO 400
C
            b1 = b1 - 1.D0/epsj
            b2 = b2 - (1.D0 - rele/epsj)
            a1 = a1 + 1.D0/epsj**3
            a2 = a2 + 1.D0*rele/epsj**3
         ENDIF
      ENDDO
      Chi = ABS(b1) + ABS(b2)
      IF(Chi.LE.EPS)THEN
         SFN = SFN + Chi
         GOTO 400
      ENDIF
C
      det = 0.5D0*(gap2*a1*a1 + a2*a2)
      IF(ABS(det).LT.1.D-10)det = SIGN(1.D-3, det)
      dellam = 0.5D0*(a1*b2 + a2*b1)/det
      delgap = (a2*b2 - gap2*a1*b1)/det
      IF(dllast*dellam.LT.0.D0 .AND. ABS(dellam).GT.ABS(dllast))THEN
         dellam = -0.5D0*dllast
         delgap = -0.5D0*delgap*dllast/dellam
      ENDIF
      dllast = dellam
      IF(ABS(delgap).GT.delmax)delgap = SIGN(delmax, delgap)
      IF(ABS(dellam).GT.delmax)dellam = SIGN(delmax, dellam)
      gap2 = gap2 + delgap
      lamdai = lamdai + dellam
      IF(iter.LE.ITERMAX)THEN
C
         IF(gap2.LT.0.D0)gap2 = 0.5D0*(gap2 - delgap)
         IF(iter.LE.ITERMAX)GOTO 300
      ENDIF
C
C
 400  IF(gap2.LT.0.05)THEN
         IF(Chi.GT.0.001)i = i + 1
         NER = NER + 1
         ier = ABS(3.0 - LOG10(Chi))
         IF(ier.LT.1)ier = 1
         IF(ier.GT.25)ier = 25
         IERm(ier) = IERm(ier) + 1
         SFEr = SFEr + Chi
         Gap = 0.D0
         Lamda = 0.5D0*(ELV(fermi) + ELV(fermi + 1))
      ELSE
         Gap = SQRT(ABS(gap2))
         Lamda = lamdai
      ENDIF
C
      END
C
      FUNCTION ENERGY(Gap, Lamda)
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 DIM
      PARAMETER(DIM = 500)
      INTEGER*4 JA, JP, NLV, NP, NH, KINd, CALc, fermi
      INTEGER*4 OLV, i, num, MAXener
      LOGICAL DEFor
      REAL*8 GI, BE, EOG, ELV, SLV, Gap, Lamda, bcs
      REAL*8 g, rele, epsj, ENERGY, SS
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
C
      fermi = JP/2 + MOD(JP, 2)
      g = GI/DBLE(JA)
C
      IF(Gap.EQ.0.D0)THEN
C
C        FERMI GAS MODEL
C
         num = JP
         bcs = 0.D0
         DO i = 1, NLV
C
            IF(OLV(i).EQ.1)THEN
               bcs = bcs + ELV(i)
               num = num - 1
            ENDIF
C
            IF(OLV(i).EQ.2)THEN
               bcs = bcs + 2.D0*ELV(i)
               num = num - 2
            ENDIF
C
         ENDDO
C
         i = 0
 50      IF(num.NE.0)THEN
            i = i + 1
C
            IF(OLV(i).EQ.0)THEN
C
               IF(num.EQ.1)THEN
                  bcs = bcs + ELV(i)
                  num = 0
               ELSE
                  bcs = bcs + 2.D0*ELV(i)
                  num = num - 2
               ENDIF
            ENDIF
            GOTO 50
         ENDIF
      ELSE
C
C        SUPERFLUID MODEL
C
         bcs = -Gap*Gap/g
         DO i = 1, NLV
            IF(OLV(i).EQ.0)THEN
               rele = ELV(i) - Lamda
               epsj = SQRT(Gap*Gap + rele*rele)
               bcs = bcs + ELV(i)*(1.D0 - rele/epsj)
            ENDIF
            IF(OLV(i).EQ.1)bcs = bcs + ELV(i)
            IF(OLV(i).EQ.2)bcs = bcs + 2.D0*ELV(i)
         ENDDO
      ENDIF
C
      ENERGY = bcs
      END
C
      SUBROUTINE SPL
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*2 DIM
      PARAMETER(DIM = 500)
      CHARACTER*2 SMAT, type*1
      CHARACTER*10 text1(2), name*12, title*80
      CHARACTER*69 text2( - 1:1)
      INTEGER*2 i, j, k, itmp
      INTEGER*4 JA, JP, jz, NLV, NP, NH, KINd, CALc, fermi, nlvtmp, 
     &          fermit
      INTEGER*4 OLV, MAXener
      LOGICAL DEFor, DEBug
      REAL*8 GI, BE, EOG, ELV, SLV, stmp, par, SS
      REAL*8 ENGrgas, ENGrbcs
      COMMON /GROUND/ ENGrgas, ENGrbcs, DEBug
C
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
C
      DATA text1/' Neutrons ', ' Protons  '/
      DATA text2/' Equidistant spacing model(ESM)            ', 
     &     ' Shell Model spectra calculation           ', 
     &     ' ESM with spacing from SHELL model         '/
C
      READ(1, '(A1)')type
      IF(type.EQ.'N' .OR. type.EQ.'n')THEN
         KINd = 1
      ELSE
         KINd = 2
      ENDIF
C
      READ(1, *)NP, NH
      READ(1, *)JA, jz
      READ(1, *)NLV, BE
C
C     CALC = 0 =>ESM with Eog from input
C     1 =>SHELL spectrum
C     2 =>ESM with Eog from SHELL spectrum
C
C     Eog  =  INPUT SINGLE-PARTCILE SPACING
C     ITMP = 0 (NO DEBUG)
C
      READ(1, *, ERR = 100)CALc, EOG, itmp
 100  READ(1, *)GI
      READ(1, *)MAXener
C
      DEBug = .FALSE.
      IF(itmp.EQ.1)DEBug = .TRUE.
C
      IF(KINd.EQ.1)THEN
         JP = JA - jz
      ELSE
         JP = jz
      ENDIF
      k = CALc - 1
      IF(k.GT.1)k = 1
      IF(k.LT. - 1)k = -1
C     K=-1 => ' Equidistant model( Spacing from input file)',
C     K=0  => ' Shell Model without equidistant levels          '
C     K=+1 => ' Equidistant model with spacing from SHELL model '
      WRITE(3, 99008)text2(k), text1(KINd), SMAT(jz), JA, JA, jz
      WRITE(3, 99013)NH, NP
      WRITE(*, 99008)text2(k), text1(KINd), SMAT(jz), JA, JA, jz
      WRITE(*, 99013)NH, NP
C
C
C
      IF(CALc.NE. - 1)THEN
C
C        SHELL-SPECTR input
C
         READ(1, '(A)')name
         OPEN(2, FILE = name, STATUS = 'OLD', ERR = 400)
         WRITE(3, 99001)
99001    FORMAT(15('-'), ' Input Data of Shell Model Spectr ', 15('-'))
         READ(2, 99012)title
C        Changed by RCN, 04/2000 to read in free format
C        read(2,1086)nlvtmp
         READ(2, *)nlvtmp
C
         IF(KINd.EQ.2)THEN
            DO i = 1, nlvtmp
               READ(2, 99012)title
            ENDDO
C           Changed by RCN, 04/2000 to read in free format
C           read(2,1086)nlvtmp
            READ(2, *)nlvtmp
         ENDIF
C
         IF(nlvtmp.LT.NLV)THEN
            WRITE(*, *)'  You ask more states than exist in spectr file'
            STOP
         ENDIF
         DO i = 1, nlvtmp
C           Read(2,1111) Elv(I),STMP,PAR
C           Changed by RCN, 04/2000 to read in free format
C           Read(2,1111) Elv(I),STMP,PAR
            READ(2, *)ELV(i), stmp, par
            SLV(i) = par*stmp
         ENDDO
         CLOSE(2)
         DEFor = .TRUE.
         DO i = 2, NLV
            IF(ELV(i).EQ.ELV(i - 1))THEN
               DEFor = .FALSE.
               GOTO 200
            ENDIF
         ENDDO
C
C        EQUIDISTANT SPACING MODEL
C
      ELSEIF(EOG.EQ.0.D0)THEN
         STOP 'ESM selected and Eog = 0'
      ENDIF
C
 200  IF(CALc.NE. - 1)EOG = DBLE(NLV)/(ELV(NLV) - ELV(1))
C
C     D = 2./Eog
C
      WRITE(3, 99002)2.D0/EOG
99002 FORMAT('      Spacing = ', f10.3, ' MeV')
C
      WRITE(3, 99014)
      IF(CALc.GT.1)THEN
         ELV(1) = 0.D0
         DO i = 2, NLV
            ELV(i) = ELV(i - 1) + 2.D0/EOG
            SLV(i) = 0.5
         ENDDO
      ENDIF
C
      fermi = JP/2 + MOD(JP, 2)
      fermit = fermi
 300  IF(ELV(fermi).EQ.ELV(fermi + 1))THEN
         fermi = fermi + 1
         GOTO 300
      ENDIF
C
      IF(BE.EQ.0.D0)THEN
         BE = ELV(NLV) - ELV(fermi)
      ELSE
         i = fermi
 350     i = i + 1
         IF(ELV(i) - ELV(fermi).LE.BE .AND. i.LT.NLV)GOTO 350
         NLV = i
      ENDIF
C
C     CHECK FOR THE SPIN (HALF OR INTEGER); SS
C
      SS = 1.0
      IF(MOD((NP+NH), 2).EQ.1)SS = 0.5
C
      DO i = 1, NLV, 4
         WRITE(3, 99003)(j, ELV(j), SLV(j), j = i, MIN(i + 3, NLV))
99003    FORMAT(4(1x, I3, 1x, F7.3, 1x, F4.1, ' |'))
      ENDDO
C
      WRITE(3, 99014)
      WRITE(3, 99004)fermit, fermi, ELV(fermi)
99004 FORMAT(/10x, 'True Fermi Level Number=', I3, /, 10x, 
     &       'Shifted Fermi Level Number=', I3, /, 10x, 
     &       'Fermi Level Energy=', F10.5, ' MeV'/)
C
      IF(DEFor)THEN
         WRITE(3, 99005)
99005    FORMAT(1X, /, 10X, ' NUCLEUS IS TREATED AS DEFORMED'/)
      ELSE
         WRITE(3, 99006)
99006    FORMAT(1X, /, 10X, ' NUCLEUS IS TREATED AS SPHERICAL'/)
      ENDIF
      WRITE(3, 99007)BE, NLV
99007 FORMAT(' Gap of energy relative to Fermi level', /, 
     &       '    below which orbitals are considered=', F10.5, ' MeV', 
     &       /, ' Number of orbitals wich are considered=', i4, //)
C
C     MaxEner=1.2*BE
C
      RETURN
 400  STOP 'SPECTRUM FILE NOT FOUND'
99008 FORMAT(/'  CALCULATION FOR ', A69, /'  FOR', A10, ' of NUCLEUS ', 
     &       A2, '-', i3, '    A=', I3, ' Z=', I3/)
99009 FORMAT(3(1x, I3, '!', 3x, F8.3, 2x, I3, 4x))
99010 FORMAT(3I10)
99011 FORMAT(E15.5, F10.1, F5.0)
99012 FORMAT(A80)
99013 FORMAT('  Configuration with ', i2, ' Holes and ', i2, 
     &       ' Particles'//)
99014 FORMAT(80('-'))
      END
C
      SUBROUTINE OUTPUT
C     Implicit Logical*1(A-Z)
      INTEGER*2 DIM
      PARAMETER(DIM = 500)
      INTEGER*4 i, j, k, fermi, JA, JP, NLV, NP, NH, KINd, CALc, nc
      INTEGER*4 OLV, MAXener, SHM, DENbcs, DENgas, jz, NCOnfig
C     REAL*4 GIFFF
      LOGICAL DEFor, DEBug
      REAL*8 GI, BE, EOG, ELV, SLV, EMAx, EHM, ENGrgas, ENGrbcs
      REAL*4 ers(101), totsd, totd, asa, eom(40, 2), parm(40), eop, eoms
      REAL*4 sig(40, 2), spin(30), energ(40), ero(40, 2)
      DIMENSION save(2), saven(2), savena(2), trc(2), slope(2)
      INTEGER*4 iel, iem, ie, ik, ili, kmax2
      CHARACTER*7 FILename
      CHARACTER*2 SMAT
      INTEGER*4 NEN, NER, IERm
      REAL*8 SFN, SFEr, SS
      COMMON /ERR   / NEN, NER, SFN, SFEr, IERm(25)
      COMMON /CANT_CONF/ NCOnfig
      COMMON /FILE  / FILename
      COMMON /GROUND/ ENGrgas, ENGrbcs, DEBug
      COMMON /DATA  / JA, JP, NLV, GI, NP, NH, KINd, BE, EOG, CALc, 
     &                MAXener, DEFor, SS
      COMMON /SPLV  / ELV(DIM), SLV(DIM), OLV(DIM)
      COMMON /LIMIT / EMAx, EHM, SHM(101), DENbcs(400, 30, 2), 
     &                DENgas(400, 30, 2)
C
      SFN = SFN/(NEN - NER)
      IF(NER.NE.0)SFEr = SFEr/NER
      WRITE(3, 99001)NEN, SFN, NER, SFEr
99001 FORMAT(1X/1X, I8, ' MINIMIZATION ENTRIES'/9X, 
     &       ' AVERAGE CHI.SQ. FOR NORMAL SOLUTIONS=', G15.5, //1X, I8, 
     &       ' ENTRIES WITHOUT REAL SOLUTIONS', /9X, 
     &       ' AVERAGE CHI.SQ. FOR UNREAL SOLUTIONS', G15.5)
      IF(NER.GT.0)THEN
         WRITE(3, 99002)
99002    FORMAT(1X, /, 3X, 'CHI.SQUARE SPECTRUM OF UNREAL SOLUTIONS'/)
         DO i = 1, 25
            j = 3 - i
            WRITE(3, 99003)j, IERm(i)
99003       FORMAT(15X, '1.0E', I3, I15)
         ENDDO
      ENDIF
C
      totsd = 0.0
      kmax = 40
      iel = 0
      iem = 0
C
C     PRINTOUT OF BCS ENERGY SHIFTS
C
      DO i = 1, 101
         IF(i.LE.100)THEN
            ers(i) = -4.95 + 0.1*FLOAT(i)
            IF(iel.EQ.0 .AND. SHM(i).NE.0.0)iel = i
            IF(SHM(i).NE.0.0)iem = i
            totsd = totsd + SHM(i)
         ENDIF
      ENDDO
      totd = totsd + SHM(101)
      IF(iel.GT.1)iel = iel - 1
      IF(iel.GT.1)iel = iel - 1
      IF(iem.LT.100)iem = iem + 1
      ie = iem - iel + 1
      DO i = 1, ie
         ers(i) = ers(i + iel - 1)
         SHM(i) = SHM(i + iel - 1)
      ENDDO
      ers(ie + 1) = 0.0
      SHM(ie + 1) = 0.0
      WRITE(6, 99005)
      WRITE(3, 99005)
      WRITE(6, 99006)(ers(i), SHM(i), i = 1, ie)
      WRITE(3, 99006)(ers(i), SHM(i), i = 1, ie)
      DO i = 1, ie
         ers(i) = ers(i) + 0.05
      ENDDO
      IF(totsd.GT.0)EHM = EHM/totsd
      IF(GI.EQ.0.0)EHM = 0.0
      WRITE(6, 99007)EHM
      WRITE(3, 99007)EHM
      WRITE(6, 99008)totd
      WRITE(3, 99008)totd
      WRITE(6, 99009)NCOnfig
      WRITE(3, 99009)NCOnfig
C
      IF(totd.NE.0)THEN        ! NO BCS STATES , only FERMI GAS CALC.
C
         IF(KINd.EQ.1)THEN
            jz = JA - JP
         ELSE
            jz = JP
         ENDIF
C
         fermi = JP/2 + MOD(JP, 2)
C
C        These lines are intended to prepare an input binary file
C        for convolution code.
C        open(21,file=filename//'.con',form='unformatted')
C        if(defor)kind=-kind
C        GIFFF=GI
C        write(21)Ja,Jz,NP,NH,Kind,KMAX*10,GIFFF,FLOAT(Nlv-Fermi),
C        >     (((Float(DenBCS(I,J,K)),J=1,30),I=1,KMAX*10),K=1,2)
C        Close(21)
C
C        open(21,file=filename//'.cof',form='unformatted')
C        if(defor)kind=-kind
C        GIFFF=GI
C        write(21)Ja,Jz,NP,NH,Kind,KMAX*10,GIFFF,FLOAT(Nlv-Fermi),
C        >     (((Float(DenGas(I,J,K)),J=1,30),I=1,KMAX*10),K=1,2)
C        Close(21)
C
         OPEN(18, FILE = FILename//'.nav')
         DO i = 1, MAXener*10
            d3 = 0.D0
            d4 = 0.D0
            DO j = 1, 30
               DO k = 1, 2
                  d3 = d3 + DENgas(i, j, k)
                  d4 = d4 + DENbcs(i, j, k)
               ENDDO
            ENDDO
            IF(d3.EQ.0.D0)d3 = 0.1D0
            IF(d4.EQ.0.D0)d4 = 0.1D0
            WRITE(18, '(1x,F6.2,1X,2(F14.2,1x))')(i - 1)*0.1, d3, d4
            WRITE(18, '(1x,F6.2,1X,2(F14.2,1x))')i*0.1, d3, d4
         ENDDO
         CLOSE(18)
C
         ntot = 0
         DO k = 1, 2
            neg_lev = 0
            WRITE(3, 99010)
            IF(k.EQ.1)WRITE(3, 99011)
            IF(k.EQ.2)WRITE(3, 99012)
            WRITE(3, 99010)
C
C           TRANSFORMATION FROM THE 0.1 MEV TO 1.0 MEV ENERGY BIN
C
            DO j = 1, 30
               DO i = 1, 40
                  IF(j.EQ.1)THEN
                     eom(i, k) = 0
                     ero(i, k) = 0
                     sig(i, k) = 0
                  ENDIF
                  ik = i - 1
                  asa = 0.0
                  asa1 = 0.0
                  DO ili = 1, 10
                     asa1 = asa1 + DENgas(ik*10 + ili, j, k)
                     asa = asa + DENbcs(ik*10 + ili, j, k)
                  ENDDO
C                 DenGas(I,J,K)=ASA1*0.1  ! CORRECTION 03/97
C                 302 DenBCS(I,J,K)=ASA *0.1  !
                  DENgas(i, j, k) = asa1
                  DENbcs(i, j, k) = asa
               ENDDO
            ENDDO
C
C           STATE DENSITY CALCULATION
C
            nc = 0
            DO i = 1, kmax
               eom(i, k) = 0.
               DO j = 1, 30
                  IF(DENbcs(i, j, k).GT.0)THEN
                     eom(i, k) = eom(i, k) + DENbcs(i, j, k)
                     nc = nc + DENbcs(i, j, k)
                  ENDIF
               ENDDO
            ENDDO
            ntot = ntot + nc
C
C           MAXIMUM ENERGY OF THE CALCULATIONS
C
            IF(k.EQ.1)kmax2 = 40
            IF(k.EQ.2)kmax2 = MIN(kmax2, 40)
            DO j = kmax, 1, -1
               IF(eom(j, k).NE.0.)THEN
                  IF(k.EQ.1)kmax2 = j
                  IF(k.EQ.2)kmax2 = MAX(kmax2, j)
                  GOTO 20
               ENDIF
            ENDDO
C
 20         IF(nc.EQ.0)THEN
               WRITE(3, 99013)
               GOTO 50
            ENDIF
C
            IF(k.EQ.2)THEN
C
C
C              CALCULATION OF THE +/ALL PARITY RATIO
C
               DO i = 1, kmax2
                  parm(i) = 100.0
                  eop = eom(i, 1)
C                 IF(EOP.EQ.1.0) EOP=0.0
                  eoms = eop + eom(i, 2)
                  IF(eoms.NE.0.0)parm(i) = eop/eoms
               ENDDO
            ENDIF
            DO i = 1, 30
               spin(i) = i - SS
            ENDDO
            DO i = 1, kmax
               sig(i, k) = 0.0
               energ(i) = i - 0.5
            ENDDO
            DO i = 1, kmax
               IF(eom(i, k).NE.0.0)THEN
C
                  sum = 0.0
C
C                 CALCULATION OF THE SPIN CUT-OFF FACTORS
C
                  DO j = 1, 30
                     IF(DENbcs(i, j, k).GE.0.0)THEN
                        sum = sum + spin(j)**2*DENbcs(i, j, k)
                     ELSE
                        sig(i, k) = 0.0
                        GOTO 40
                     ENDIF
                  ENDDO
                  IF(eom(i, k).GT.0.)THEN
                     sig(i, k) = sum/eom(i, k)
                     sig(i, k) = SQRT(sig(i, k))
                  ELSE
                     sig(i, k) = 0
                  ENDIF
               ENDIF
 40         ENDDO
C
C           S T A T E    D E N S I T Y     P R I N T O U T
C
            WRITE(3, 99014)
            WRITE(3, 99016)(energ(i), eom(i, k), sig(i, k), (DENbcs(i,j,
     &                     k), j = 1, 15), i = 1, kmax2)
            WRITE(3, 99017)((DENbcs(i,j,k), j = 16, 30), i = 1, kmax2)
            IF(.NOT.(DEFor))THEN
C
C              CALCULATION OF THE LEVEL DENSITY FROM STATE DENSITY
C
               DO j = 1, kmax2
                  ero(j, k) = 0.0
                  IF(SS.EQ.1.0)DENbcs(j, 1, k) = DENbcs(j, 1, k)*2.
                  DO i = 1, 29
                     DENbcs(j, i, k) = (DENbcs(j, i, k) - DENbcs(j, i + 
     &                                 1, k))/2.0
                     IF(DENbcs(j, i, k).LT.0)THEN
                        neg_lev = neg_lev + 1
                        DENbcs(j, i, k) = 0
                     ENDIF
C
                     ero(j, k) = ero(j, k) + DENbcs(j, i, k)
                  ENDDO
               ENDDO
C
C
C              L E V E L    D E N S I T Y     P R I N T O U T
C
               WRITE(3, 99015)
               WRITE(3, 99016)(energ(i), ero(i, k), sig(i, k), (DENbcs(i
     &                        ,j,k), j = 1, 15), i = 1, kmax2)
               WRITE(3, 99017)((DENbcs(i,j,k), j = 16, 30), i = 1, 
     &                        kmax2)
               WRITE(3, 99018)neg_lev
            ENDIF
            IF(k.EQ.2)THEN
C
C              +/ALL PARITY RATIO PRINTOUT
C
               WRITE(3, 99010)
               WRITE(3, 99019)(energ(i), parm(i), i = 1, kmax2)
            ENDIF
C
C           SPIN CUT-0FF FACTOR ENERGY DEPENDENCE ANALYSIS
C
            nn = 0
            enes2 = 0.0
            enes = 0.0
            sigs = 0.0
            smix = 0.0
            DO i = 1, kmax2
               IF(sig(i, k).NE.0.0)THEN
                  nn = nn + 1
                  enes2 = enes2 + energ(i)**2
                  enes = enes + energ(i)
                  sigs = sigs + sig(i, k)
               ENDIF
               smix = smix + sig(i, k)*energ(i)
            ENDDO
            IF(nn.GT.1)THEN
               save(k) = sigs/nn
               eave = enes/nn
               stde = enes2/nn - eave**2
               IF(stde.GT.0.0)THEN
                  slope(k) = smix/nn - eave*save(k)
                  slope(k) = slope(k)/stde
                  trc(k) = save(k) - slope(k)*eave
                  saven(k) = save(k)**2/(NP + NH)
                  savena(k) = saven(k)/(FLOAT(JA)**0.666666)
                  WRITE(3, 99020)
                  WRITE(3, 99021)save(k), saven(k), savena(k), trc(k), 
     &                           slope(k)
               ENDIF
            ENDIF
 50      ENDDO
C
         IF(KINd.EQ.1)THEN
            WRITE(3, 99026)SMAT(JA - JP), JA, NLV, NLV
         ELSE
            WRITE(3, 99026)SMAT(JP), JA, NLV, NLV
         ENDIF
         WRITE(3, 99027)GI/JA, GI
C
         IF(ntot.GT.0)THEN
C
            ipp = kmax2
            IF(ipp.LT.ie)ipp = ie
            IF(ipp.GT.kmax)ipp = kmax
            DO i = ie, 100
               SHM(i) = 0.0
               ers(i) = 0.0
            ENDDO
C
            WRITE(3, 99022)
            WRITE(3, 99023)(energ(i), eom(i, 1), eom(i, 2), ero(i, 1), 
     &                     ero(i, 2), sig(i, 1), sig(i, 2), parm(i), 
     &                     ers(i), SHM(i), i = 1, ipp)
C
            OPEN(8, FILE = FILename//'.dat')
C           Commented, not needed RCN, 04/2000
C           Open(18,File=FileName//'.nav')
            ntot = 0
            ntotl = 0
            DO i = 1, kmax2
C              D3 = STATE DENSITY
C              D4 = LEVEL DENSITY
               d3 = eom(i, 1) + eom(i, 2)
               d4 = ero(i, 1) + ero(i, 2)
               ntot = ntot + NINT(d3)
               ntotl = ntotl + NINT(d4)
               IF(d3.EQ.0.D0)d3 = 0.1D0
               IF(d4.EQ.0.D0)d4 = 0.1D0
               WRITE(8, 99030)(i - 1), d4, d3, MAX(REAL(ntot), 1.)
               WRITE(8, 99030)i, d4, d3, MAX(REAL(ntot), 1.)
            ENDDO
C***************************************************
            CLOSE(8)
         ELSE
            ntotl = 0
         ENDIF
C
         WRITE(3, 99024)ntot, ntotl
         WRITE(6, 99024)ntot, ntotl
      ENDIF
C
      WRITE(3, 99004)
99004 FORMAT(1X, //10x, 
     &       '************************************************'/10x, 
     &       '************* FERMI GAS RESULTS ****************'/10x, 
     &       '************************************************')
C
      DO k = 1, 2
         neg_lev = 0
         IF(k.EQ.1)WRITE(3, 99011)
         IF(k.EQ.2)WRITE(3, 99012)
         WRITE(3, 99010)
C
         nc = 0
         DO i = 1, 40
            eom(i, k) = 0.
            ero(i, k) = 0.
            DO j = 1, 30
               IF(DENgas(i, j, k).GT.0)THEN
                  eom(i, k) = eom(i, k) + DENgas(i, j, k)
                  nc = nc + DENgas(i, j, k)
               ENDIF
            ENDDO
         ENDDO
C
         ntot = ntot + nc
C
         IF(ntot.EQ.0 .AND. k.EQ.2)STOP 
     &                          'NO STATES at all FOR THESE PARAMETERS '
C
C        MAXIMUM ENERGY OF THE CALCULATIONS
C
         IF(k.EQ.1)kmax2 = 40
         IF(k.EQ.2)kmax2 = MIN(kmax2, 40)
         DO j = kmax, 1, -1
            IF(eom(j, k).NE.0.)THEN
               IF(k.EQ.1)kmax2 = j
               IF(k.EQ.2)kmax2 = MAX(kmax2, j)
               GOTO 100
            ENDIF
         ENDDO
C
 100     IF(nc.EQ.0)THEN
            WRITE(3, 99013)
            GOTO 200
         ENDIF
C
         IF(k.EQ.2)THEN
C
C
C           CALCULATION OF THE +/ALL PARITY RATIO
C
            DO i = 1, kmax2
               parm(i) = 100.0
               eop = eom(i, 1)
C              IF(EOP.EQ.1.0) EOP=0.0
               eoms = eop + eom(i, 2)
               IF(eoms.NE.0.0)parm(i) = eop/eoms
            ENDDO
         ENDIF
         DO i = 1, 30
            spin(i) = i - SS
         ENDDO
         DO i = 1, 40
            sig(i, k) = 0.0
            energ(i) = i - 0.5
         ENDDO
         DO i = 1, 40
            IF(eom(i, k).NE.0.0)THEN
               sum = 0.0
C
C              CALCULATION OF THE SPIN CUT-OFF FACTORS
C
               DO j = 1, 30
                  IF(DENgas(i, j, k).GE.0.0)THEN
                     sum = sum + spin(j)**2*DENgas(i, j, k)
                  ELSE
                     sig(i, k) = 0.0
                     GOTO 150
                  ENDIF
               ENDDO
               IF(eom(i, k).GT.0.)THEN
                  sig(i, k) = sum/eom(i, k)
                  sig(i, k) = SQRT(sig(i, k))
               ELSE
                  sig(i, k) = 0
               ENDIF
            ENDIF
 150     ENDDO
C
C        S T A T E    D E N S I T Y     P R I N T O U T
C
         WRITE(3, 99014)
         WRITE(3, 99016)(energ(i), eom(i, k), sig(i, k), (DENgas(i,j,k), 
     &                  j = 1, 15), i = 1, kmax2)
         WRITE(3, 99017)((DENgas(i,j,k), j = 16, 30), i = 1, kmax2)
         IF(.NOT.(DEFor))THEN
C
C           CALCULATION OF THE LEVEL DENSITY FROM STATE DENSITY
C
            DO j = 1, kmax2
               ero(j, k) = 0.0
               IF(SS.EQ.1.0)DENgas(j, 1, k) = DENgas(j, 1, k)*2.
               DO i = 1, 29
                  DENgas(j, i, k) = (DENgas(j, i, k) - DENgas(j, i + 1, 
     &                              k))/2.0
                  IF(DENgas(j, i, k).LT.0)THEN
                     neg_lev = neg_lev + 1
                     DENgas(j, i, k) = 0
                  ENDIF
                  ero(j, k) = ero(j, k) + DENgas(j, i, k)
               ENDDO
            ENDDO
C
C
C           L E V E L    D E N S I T Y     P R I N T O U T
C
            WRITE(3, 99015)
            WRITE(3, 99016)(energ(i), ero(i, k), sig(i, k), (DENgas(i,j,
     &                     k), j = 1, 15), i = 1, kmax2)
            WRITE(3, 99017)((DENgas(i,j,k), j = 16, 30), i = 1, kmax2)
            WRITE(3, 99018)neg_lev
         ENDIF
         IF(k.EQ.2)THEN
C
C           +/ALL PARITY RATIO PRINTOUT
C
            WRITE(3, 99010)
            WRITE(3, 99019)(energ(i), parm(i), i = 1, kmax2)
         ENDIF
C
C        SPIN CUT-0FF FACTOR ENERGY DEPENDENCE ANALYSIS
C
         nn = 0
         enes2 = 0.0
         enes = 0.0
         sigs = 0.0
         smix = 0.0
         DO i = 1, kmax2
            IF(sig(i, k).NE.0.0)THEN
               nn = nn + 1
               enes2 = enes2 + energ(i)**2
               enes = enes + energ(i)
               sigs = sigs + sig(i, k)
            ENDIF
            smix = smix + sig(i, k)*energ(i)
         ENDDO
         IF(nn.GT.1)THEN
            save(k) = sigs/nn
            eave = enes/nn
            stde = enes2/nn - eave**2
            IF(stde.GT.0.)THEN
               slope(k) = smix/nn - eave*save(k)
               slope(k) = slope(k)/stde
               trc(k) = save(k) - slope(k)*eave
               saven(k) = save(k)**2/(NP + NH)
               savena(k) = saven(k)/(FLOAT(JA)**0.666666)
               WRITE(3, 99020)
               WRITE(3, 99021)save(k), saven(k), savena(k), trc(k), 
     &                        slope(k)
            ENDIF
         ENDIF
 200  ENDDO
C
      OPEN(8, FILE = FILename//'.fer')
      ntot = 0
      ntotl = 0
      DO i = 1, kmax2
C        D3 = STATE DENSITY
C        D4 = LEVEL DENSITY
         d3 = eom(i, 1) + eom(i, 2)
         d4 = ero(i, 1) + ero(i, 2)
         ntot = ntot + NINT(d3)
         ntotl = ntotl + NINT(d4)
         IF(d3.EQ.0.D0)d3 = 0.1D0
         IF(d4.EQ.0.D0)d4 = 0.1D0
C        Substituting RCN, 04/2000
C        Write(8,1001) (I-1),D4,D3
C        Write(8,1001) I,D4,D3
C        Printing total number of levels (STATES)
         WRITE(8, 99030)(i - 1), d4, d3, MAX(REAL(ntot), 1.)
         WRITE(8, 99030)i, d4, d3, MAX(REAL(ntot), 1.)
      ENDDO
C***************************************************
      CLOSE(8)
C
      WRITE(3, 99025)ntot, ntotl
      WRITE(6, 99025)ntot, ntotl
C
      CLOSE(6)
C
      IF(KINd.EQ.1)THEN
         WRITE(3, 99026)SMAT(JA - JP), JA, NLV, NLV
      ELSE
         WRITE(3, 99026)SMAT(JP), JA, NLV, NLV
      ENDIF
      WRITE(3, 99027)GI/JA, GI
C
      ipp = kmax2
      IF(ipp.LT.ie)ipp = ie
      IF(ipp.GT.40)ipp = 40
      DO i = ie, 100
         SHM(i) = 0.0
         ers(i) = 0.0
      ENDDO
C
      WRITE(3, 99022)
      WRITE(3, 99023)(energ(i), eom(i, 1), eom(i, 2), ero(i, 1), 
     &               ero(i, 2), sig(i, 1), sig(i, 2), parm(i), ers(i), 
     &               SHM(i), i = 1, ipp)
      WRITE(3, 99021)save(1), saven(1), savena(1), trc(1), slope(1)
      WRITE(3, 99021)save(2), saven(2), savena(2), trc(2), slope(2)
99005 FORMAT(1X, //, 3X, 'HISTOGRAM OF ENERGY SHIFTS ', /, 11X, 'MEV', 
     &       8X, 'BCS-FG', /)
99006 FORMAT(3X, F6.2, 2X, I9)
99007 FORMAT(1X, /, 3X, 'AVERAGE ENERGY SHIFT BCS-FG=', F6.2, /)
99008 FORMAT(3X, 'TOTAL NUMBER OF STATES(BCS)=', G15.5, /)
99009 FORMAT(3X, 'TOTAL NUMBER OF CONFIGURATIONS=', I10/)
99010 FORMAT(1X, //)
99011 FORMAT(/20X, 'P O S I T I V E   P A R I T Y')
99012 FORMAT(/20X, 'N E G A T I V E   P A R I T Y')
99013 FORMAT(/20X, 'NO STATES WITH THIS PARITY'/)
99014 FORMAT(3X/30X, ' ENERGY AND M DEPENDENCE OF STATE DENSITIES'//3X, 
     &       'E', 4X, 'OMEGA(E,PI)  ', 'SIGMA', 18X, ' OMEGA(E,M,PI)')
99015 FORMAT(3X//30X, 'ENERGY AND SPIN DEPENDENCE OF LEVEL DENSITIES'//
     &       3X, 'E', 4X, '  RHO(E,PI)  ', 'SIGMA', 18X, 
     &       '   RHO(E,J,PI)')
99016 FORMAT(F5.1, 1X, E13.6, F6.2, 1X, 15I7)
99017 FORMAT(1X, /, (26X, 15I7))
99018 FORMAT(/5X, ' NUMBER OF LEVELS WITH NEGATIVE DENSITY=', I8/)
99019 FORMAT(1X, 'POS. PARITY STATES/ALL STATES', //, 
     &       (5X, F4.1, 2X, F5.3))
99020 FORMAT(1X, //, 10X, 'SPIN CUT-OFF ENERGY DEPENDENCE ANALYSIS')
99021 FORMAT(1X, /, 10X, 'SIG AVE.=', F6.3, ' SIG**2/N=', F6.3, 
     &       ' SIG**2/N/A**2/3=', F6.3, /, 10X, 'SIG**2=', F6.3, '+', 
     &       F6.3, '*E')
99022 FORMAT(2X, 'MEV ', 3X, '+ST. DENS.', 3X, '-ST. DENS.', 3X, 
     &       '+LEV. DENS.', 2X, '-LEV. DENS.', 1X, '+SIG.', 1X, '-SIG.', 
     &       1X, '+/ALL  ', '  MEV ', '   SHIFT', /)
99023 FORMAT(1X, F4.1, 1X, 4E13.5, 2F6.2, 2X, F5.3, 2X, F5.2, I9.0)
99024 FORMAT(/1X, 'TOTAL NUMBER OF STATES(BCS) = ', I10/1X, 
     &       'TOTAL NUMBER OF LEVELS(BCS) = ', I10)
99025 FORMAT(/1X, 'TOTAL NUMBER OF STATES(Fermi Gas) = ', I10/1X, 
     &       'TOTAL NUMBER OF LEVELS(Fermi Gas) = ', I10)
99026 FORMAT(//25X, ' NUCLEUS ', A2, I3/, 25X, 15('-'), //10X, I3, 
     &       ' NEUTRON OR', I4, ' PROTON LEVELS TAKEN INTO ACCOUNT')
99027 FORMAT(/10X, 'G=', F10.5, ' CORRESPONDS TO', F6.2, '/A'//)
99028 FORMAT(10X, I1, ' PARTICLES', I4, ' HOLES CONFIGURATIONS', //)
99029 FORMAT(6(1x, E12.5))
99030 FORMAT(1x, I4, 4(1x, F14.2))
      END
C
      FUNCTION SMAT(I)
      CHARACTER*2 mat(99), SMAT
      DATA mat/' H', 'He', 'Li', 'Be', ' B', ' C', ' N', ' O', ' F', 
     &     'Ne', 'Na', 'Mg', 'Al', 'Si', ' P', ' S', 'Cl', 'Ar', ' K', 
     &     'Ca', 'Sc', 'Ti', ' V', 'Cr', 'Mn', 'Fe', 'Co', 'Ni', 'Cu', 
     &     'Zn', 'Ga', 'Ge', 'As', 'Se', 'Br', 'Kr', 'Rb', 'Sr', ' Y', 
     &     'Zr', 'Nb', 'Mo', 'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 
     &     'Sn', 'Sb', 'Te', ' I', 'Xe', 'Cs', 'Ba', 'La', 'Ce', 'Pr', 
     &     'Nd', 'Pm', 'Sm', 'Eu', 'Gd', 'Tb', 'Dy', 'Ho', 'Er', 'Tm', 
     &     'Yb', 'Lu', 'Hf', 'Ta', ' W', 'Re', 'Os', 'Ir', 'Pt', 'Au', 
     &     'Hg', 'Tl', 'Pb', 'Bi', 'Po', 'At', 'Rn', 'Fr', 'Ra', 'Ac', 
     &     'Th', 'Pa', ' U', 'Np', 'Pu', 'Am', 'Cm', 'Bk', 'Cf', 'Es'/
      IF(I.LT.100)THEN
         SMAT = mat(I)
      ELSE
         SMAT = 'NO'
      ENDIF
      END
C
      SUBROUTINE PNXCB(M, K, N, Mold, Mnew)
      IMPLICIT LOGICAL*1(A - Z)
      INTEGER*4 M(*), K, N, Mold, Mnew, j
C-----------------------------------------------------------------------
C-----FROM:
C-----COMBINATION GENERATION
C-----W.H.PAYNE,F.M.IVES - ACM TRANS.OF MATH.SOFT. 5,2(1979) P.163
C-----------------------------------------------------------------------
C     LIU-TANG SEQUENCE POINTER PROGRAMMED COMBINATION GENERATOR.
C     EXCHANGE MARK LOCATED AT MOLD WITH MARK LOCATED AT MNEW.
C     N MARKS TAKEN K AT A TIME. N>1. 0<K<=N. SEPTEMBER 1975.
C     M IS A POINTER ARRAY OF DIMENSION MIN(K,N-K) POINTING TO THE
C     POSITIONS OF THE MIN(K,N-K) IDENTICAL MARKS.
      j = 1
C     FOR 'DOWN-UP' SEQUENCE INSERT GO TO 20 HERE.
C     J-TH MARK IS MOVING RIGHT.
 100  Mold = M(j)
      Mnew = Mold + 1
      IF(j.NE.K)THEN
         IF(Mnew.GE.M(j + 1))THEN
C           J-TH MARK CAN'T MOVE. CONSIDER NEXT MARK.
            j = j + 1
C           J-TH MARK IS MOVING LEFT
            Mold = M(j)
            Mnew = Mold - 1
            IF(Mnew.GE.j)THEN
               M(j) = Mnew
               IF(j.EQ.1)RETURN
               Mnew = j - 1
               M(j - 1) = Mnew
               GOTO 99999
            ELSE
               IF(j.EQ.K)GOTO 200
C              J-TH MARK CAN'T MOVE. CONSIDER NEXT MARK.
               j = j + 1
               GOTO 100
            ENDIF
         ENDIF
      ELSEIF(Mnew.GT.N)THEN
C        MOVE MARK J.
C        PREVIOUS SECTION BRANCHED TO 31,32,41,OR 42.
C        DEPENDING ON TYPE OF MOVE.
C        31 MOVING RIGHT, J=K,AT END OF SEQUENCE
C        32 MOVING RIGHT, NOT AT END OF SEQUENCE
         Mnew = K
         M(j) = Mnew
         RETURN
      ENDIF
      M(j) = Mnew
      IF(j.EQ.1)RETURN
      M(j - 1) = Mold
      Mold = j - 1
      RETURN
C     41 MOVING LEFT, J=K, AT END OF SEQUENCE
C     42 MOVING RIGHT, NOT AT END OF SEQUENCE
 200  Mnew = N
      M(j) = Mnew
      RETURN
99999 END
